#!/usr/bin/env python
import asyncio
import logging
import os

from dotenv import load_dotenv
from ssc2ce import Coinbase

logging.basicConfig(format='%(asctime)s %(name)s %(funcName)s %(levelname)s %(message)s', level=logging.INFO)
logger = logging.getLogger("bitfinex-basic-example")

env_path = os.path.join(os.path.dirname(__file__), '.env')
load_dotenv(env_path)

auth_param = dict(api_key=os.environ.get('COINBASE_PRO_KEY'),
                  secret_key=os.environ.get('COINBASE_PRO_SECRET'),
                  passphrase=os.environ.get('COINBASE_PRO_PASSPHRASE'))

print(auth_param)


def handle_message(message: str):
    logger.info(message)


conn = Coinbase(auth_param=auth_param)
conn.on_message = handle_message


async def start():
    res = await conn.private_get('/accounts')
    print(res)
    await conn.stop()
    # await conn.ws.send_json({
    #     "type": "subscribe",
    #     "product_ids": [
    #         "BTC-USD",
    #         "ETH-BTC"
    #     ],
    #     "channels": [
    #         "level2",
    #         "heartbeat"
    #     ]
    # })


def handle_heartbeat(data: dict) -> bool:
    logger.info(f"{repr(data)}")
    return True


conn.on_connect_ws = start


def stop():
    asyncio.ensure_future(conn.stop())


loop = asyncio.get_event_loop()
loop.call_later(3600, stop)

try:
    loop.run_until_complete(conn.run_receiver())
except KeyboardInterrupt:
    logger.info("Application closed by KeyboardInterrupt.")



from ssc2ce.common.l2_book import L2Book


class CexL2Book(L2Book):
    def __init__(self, instrument: str):
        L2Book.__init__(self, instrument)
        self.sequence = None
        self.time = None



import os

from unittest import TestCase

from ssc2ce import CoinbaseParser
from .coinbase_expected_result import results

path = os.path.dirname(os.path.abspath(__file__))
path = os.path.join(path, "coinbase_dump.jsonl")
with open(path) as f:
    lines = f.readlines()


class TestCexParser(TestCase):
    broken = False

    def test_one(self):
        parser = CoinbaseParser()
        book = parser.get_book("BTC-EUR")
        for i, line in enumerate(lines):
            # print(results[i])
            if results[i][0] == 1:
                parser.parse(line)
                self.assertTrue(book.valid())
                self.assertEqual(book.top_bid_price(), results[i][1])
                self.assertEqual(book.top_ask_price(), results[i][2])
            else:
                parser.parse(line)
                self.assertFalse(book.valid())



#!/usr/bin/env python
import asyncio
import logging

from ssc2ce.bitfinex import Bitfinex, BitfinexL2Book

logging.basicConfig(
    format='%(asctime)s %(name)s %(funcName)s %(levelname)s %(message)s',
    level=logging.INFO)
logger = logging.getLogger("bitfinex-basic-example")

conn = Bitfinex()


class BookMaintainer:
    def __init__(self, instrument):
        self.book = BitfinexL2Book(instrument)
        self.check_sum = None
        self.active = False
        self.top_bid = 0.
        self.top_ask = 0.

    def handle_book(self, message, _):
        if type(message[1]) is str:
            if message[1] == "cs":
                pass
        elif len(message[1]) == 3:
            self.book.handle_update(message)
            self.book.time = message[-1]
        else:
            self.book.handle_snapshot(message)
            self.book.time = message[-1]

        if self.top_ask != self.book.top_ask_price() or self.top_bid != self.book.top_bid_price():
            self.top_ask = self.book.top_ask_price()
            self.top_bid = self.book.top_bid_price()
            logger.info(
                f"{self.book.instrument()} bid:{self.top_bid} ask:{self.top_ask}")


btc = BookMaintainer("BTC-USD")
eth = BookMaintainer("ETH-USD")


async def subscribe():
    await conn.subscribe({'channel': "book", 'symbol': "tBTCUSD"}, handler=btc.handle_book)
    await conn.subscribe({'channel': "book", 'symbol': "tETHUSD"}, handler=eth.handle_book)


conn.on_connect_ws = subscribe

loop = asyncio.get_event_loop()

try:
    loop.run_until_complete(conn.run_receiver())
except KeyboardInterrupt:
    loop.run_until_complete(conn.stop())
finally:
    print("Application closed")



import logging
import argparse
from time import time

from examples.common.book_watcher import BookWatcher

from ssc2ce import create_parser

logging.basicConfig(format='%(asctime)s %(name)s %(funcName)s %(levelname)s %(message)s', level=logging.INFO)
logger = logging.getLogger("deribit-parser")


class FileReader:
    def __init__(self, exchange: str, is_cpp: bool):

        self.parser = create_parser(exchange, is_cpp)
        if self.parser is None:
            exit(1)

        self.counter = 0
        self.watcher = BookWatcher(self.parser, False)

    def run(self, filename: str):
        start = time()
        i = 0
        with open(filename) as f:
            for line in f:
                i += 1
                if not self.parser.parse(line):
                    self.handle_message(line)

        logger.info(f"{i} in {time() - start} sec. {self.counter}")

    def handle_message(self, _: str) -> None:
        self.counter += 1


def main():
    parser = argparse.ArgumentParser('ssc2ce parser example.')
    parser.add_argument('exchange',
                        type=str,
                        help='...')
    parser.add_argument('file',
                        type=str,
                        help='...')
    parser.add_argument('-c', '--cpp',
                        dest='is_cpp',
                        action='store_true',
                        default=False,
                        help='...')

    args = parser.parse_args()

    FileReader(args.exchange, args.is_cpp).run(args.file)


if __name__ == '__main__':
    main()



import asyncio
import json
from ssc2ce import Deribit

conn = Deribit()

print("Is connected after init", conn.is_connected())


def close():
    print("Check connection  before closing", conn.is_connected())
    asyncio.ensure_future(conn.stop())


loop = asyncio.get_event_loop()


loop.call_later(1, close)

try:
    loop.run_until_complete(conn.run_receiver())
except KeyboardInterrupt:
    print("Application closed by KeyboardInterrupt.")


print("Is connected before exit", conn.is_connected())



import json
import logging
from typing import Callable, Dict

from ssc2ce.common.abstract_parser import AbstractParser
from ssc2ce.cex.l2_book import CexL2Book
from ssc2ce.common.exceptions import BrokenOrderbook


class CexParser(AbstractParser):
    """

    """

    def __init__(self):
        AbstractParser.__init__(self)
        self._routes: Dict[str, Callable[[dict], None]] = None
        self._handler_broken_orderbook: Callable[[str, int, int], None] = None
        self._routes = {
            # 'auth', self.handle_auth,
            # 'balance', self.handle_balance,
            # 'disconnecting', self.handle_disconnecting,
            # 'get-balance', self.handle_get_balance,
            'md_update': self.handle_md_update,
            # 'obalance', self.handle_obalance,
            # 'open-orders', self.handle_open_orders,
            # 'order', self.handle_order,
            'order-book-subscribe': self.handle_order_book_subscribe,
            # 'order-book-unsubscribe': self.handle_order_book_unsubscribe,
            # 'ping': self.handle_ping,
            # 'tick': self.handle_tick,
            # 'tx', self.handle_tx,
        }
        self.logger = logging.getLogger(__name__)
        self.books = {}

    def parse(self, message: str) -> bool:
        data = json.loads(message)
        message_type = data.get('e')
        processed: bool = False

        if message_type:
            handler = self._routes.get(message_type)
            if handler:
                handler(data)
                processed = True
        else:
            self.logger.error(message)

        return processed

    def get_book(self, instrument: str) -> CexL2Book:
        book: CexL2Book = self.books.get(instrument)
        if book is None:
            book = CexL2Book(instrument)
            self.books[instrument] = book

        return book

    def handle_order_book_subscribe(self, message: dict) -> None:
        """

        :param message:
        :return:
        """
        ok = message.get('ok')
        if ok != 'ok':
            self.logger.error(message)
            return

        data = message["data"]
        book = self.get_book(data["pair"])
        book.clear()
        book.sequence = data["id"]
        book.time = data["timestamp"]

        for price, size in data['bids']:
            book.bids.add(price, size)

        for price, size in data['asks']:
            book.asks.add(price, size)

        if self._on_book_setup:
            self._on_book_setup(book)

    def handle_md_update(self, message: dict) -> None:
        """

        :param message:
        :return:
        """

        data = message["data"]
        book = self.get_book(data["pair"])
        book.sequence += 1
        if book.sequence != data["id"]:
            if self._handler_broken_orderbook:
                self._handler_broken_orderbook(data["pair"], book.sequence - 1, data["id"])
            else:
                raise BrokenOrderbook(data["pair"], book.sequence - 1, data["id"])

        book.time = data["time"]
        for price, size in data['bids']:
            book.bids.update(price, size)

        for price, size in data['asks']:
            book.bids.update(price, size)

        if self._on_book_update and book.valid():
            self._on_book_update(book)



#!/usr/bin/env python
import asyncio
import json
from ssc2ce import Deribit

conn = Deribit()

pending = {}


def handle_instruments(data: dict):
    del pending[data["id"]]
    print("handle_instruments", json.dumps(data))
    if not pending:
        asyncio.ensure_future(subscribe())


async def get_instruments(symbol):
    request_id = await conn.get_instruments(symbol, callback=handle_instruments)
    pending[request_id] = symbol


def handle_currencies(data: dict):
    for currency in data["result"]:
        asyncio.ensure_future(get_instruments(currency["currency"]))


async def get_currencies():
    await conn.get_currencies(handle_currencies)


async def subscribe():
    await conn.send_public(request={
        "method": "public/subscribe",
        "params": {
            "channels": ["deribit_price_index.btc_usd",
                         "deribit_price_index.eth_usd"]
        }
    })


def handle_subscription(data):
    method = data.get("method")
    if method and method == "subscription":
        if data["params"]["channel"].startswith("deribit_price_index"):
            index_name = data["params"]["data"]["index_name"]
            price = data["params"]["data"]["price"]
            print(f" Deribit Price Index {index_name.upper()}: {price}")


conn.on_connect_ws = get_currencies
conn.method_routes += [("subscription", handle_subscription)]

loop = asyncio.get_event_loop()

try:
    loop.run_until_complete(conn.run_receiver())
except KeyboardInterrupt:
    print("Application closed by KeyboardInterrupt.")



#!/usr/bin/env python

import asyncio
import logging
import os
import re
from uuid import uuid4
from typing import Pattern
from dotenv import load_dotenv
from ssc2ce import Deribit, AuthType


class MyApp:
    def __init__(self):
        logging.basicConfig(
            format='%(asctime)s %(name)s %(funcName)s %(levelname)s %(message)s', level=logging.INFO)
        self.logger = logging.getLogger("deribit-private")
        self.direct_requests = {}

        dotenv_path = os.path.join(os.path.dirname(__file__), '.env')
        load_dotenv(dotenv_path)

        client_id = os.environ.get('DERIBIT_CLIENT_ID')
        client_secret = os.environ.get('DERIBIT_CLIENT_SECRET')

        if client_id is None or client_secret is None:
            self.logger.error(
                "Please setup environment variables DERIBIT_CLIENT_ID and DERIBIT_CLIENT_SECRET")
            exit(0)

        self.deribit = Deribit(client_id=client_id,
                               client_secret=client_secret,
                               auth_type=AuthType.CREDENTIALS,
                               scope=None,
                               get_id=lambda: str(uuid4()))

        self.deribit.on_handle_response = self.on_handle_response
        self.deribit.on_authenticated = self.after_login
        self.deribit.on_token = self.on_token
        self.deribit.on_response_error = self.on_response_error

        self.deribit.method_routes += [
            ("subscription", self.handle_subscription),
        ]
        self.deribit.response_routes += [
            ("public/subscribe", self.printer),
            ("private/subscribe", self.printer),
            ("private/get_position", self.printer),
            ("private/enable_cancel_on_disconnect", self.printer),
            ("private/get_account_summary", self.printer),
        ]

        self.subscription_route = [
            (re.compile(r"book.(.*).raw"), self.handle_order_book_change),
            (re.compile(r"user.trades.(.*).raw"), self.handle_user_trades),
            (re.compile(r"user.orders.(.*).raw"), self.handle_user_orders),
            (re.compile(r"deribit_price_index.(.*)"), self.handle_price_index),
            (re.compile(r"trades.(.*).raw"), self.handle_trades),
            (re.compile(r"ticker.(.*).raw"), self.handle_ticker)
        ]

    async def do_something_after_login(self):
        await self.deribit.send_private(request={
            "method": "private/get_account_summary",
            "params": {
                "currency": "BTC",
                "extended": True
            }
        })

        await self.deribit.send_private(request={
            "method": "private/get_position",
            "params": {
                "instrument_name": "BTC-PERPETUAL"
            }
        })

        await self.deribit.send_public(request={
            "method": "private/subscribe",
            "params": {
                "channels": [
                    "deribit_price_index.btc_usd",
                    "book.BTC-PERPETUAL.raw",
                    "trades.BTC-PERPETUAL.raw",
                    "user.orders.BTC-PERPETUAL.raw",
                    "user.trades.BTC-PERPETUAL.raw"
                ]
            }
        })

        await self.deribit.send_public(request={
            "method": "public/set_heartbeat",
            "params": {
                "interval": 15
            }
        })

    async def printer(self, **kwargs):
        self.logger.info(f"{repr(kwargs)}")

    @staticmethod
    def resolve_route(value, routes):
        key, handler = None, None
        for key, handler in routes:
            if key:
                if isinstance(key, str):
                    if key == value:
                        return handler
                elif isinstance(key, Pattern) and re.match(key, value):
                    return handler

        if key is not None and key == "" and handler:
            return handler

    def handle_subscription(self, data: dict):
        channel = data["params"]["channel"]
        handler = self.resolve_route(channel, self.subscription_route)
        if handler:
            return handler(data)

    def after_login(self):
        asyncio.ensure_future(self.do_something_after_login())

    async def setup_refresh(self, refresh_interval):
        await asyncio.sleep(refresh_interval)
        await self.deribit.auth_refresh_token()

    def on_token(self, params):
        refresh_interval = min(600, params["expires_in"])
        asyncio.ensure_future(self.setup_refresh(refresh_interval))

    def on_handle_response(self, data):
        request_id = data["id"]
        if request_id in self.direct_requests:
            self.logger.info(
                f"Caught response {repr(data)} to direct request {self.direct_requests[request_id]}")
        else:
            self.logger.error(
                f"Can't find request with id:{request_id} for response:{repr(data)}")

    def handle_order_book_change(self, message):
        data = message["params"]["data"]
        self.logger.debug(f"{repr(data)}")

    def handle_user_trades(self, message):
        data = message["params"]["data"]
        self.logger.info(f"{repr(data)}")

    def handle_user_orders(self, message):
        data = message["params"]["data"]
        self.logger.info(f"{repr(data)}")

    def handle_price_index(self, message):
        data = message["params"]["data"]
        index_name = data['index_name']
        self.logger.info(f"{index_name}: {repr(data)}")

    def handle_trades(self, message):
        data = message["params"]["data"]
        self.logger.debug(f"{repr(data)}")

    def handle_ticker(self, message):
        data = message["params"]["data"]
        instrument_name = data['instrument_name']
        self.logger.debug(f"{instrument_name}: {repr(data)}")

    def on_response_error(self, data):
        self.logger.error(f"Receive error {repr(data)}")
        asyncio.ensure_future(self.deribit.stop())

    def disable_heartbeat(self):
        asyncio.ensure_future(self.deribit.disable_heartbeat())

    def logout(self):
        asyncio.ensure_future(self.deribit.auth_logout())

    async def run(self):
        await self.deribit.run_receiver()

    async def stop(self):
        await self.deribit.stop()


if __name__ == '__main__':
    app = MyApp()
    loop = asyncio.get_event_loop()
    try:
        loop.run_until_complete(app.run())
    except KeyboardInterrupt:
        loop.run_until_complete(app.stop())
    finally:
        app.logger.info('Program finished')



#!/usr/bin/env python
import asyncio
import json
import logging
import sys

from examples.common.book_watcher import BookWatcher
from ssc2ce import Deribit, create_parser

conn = Deribit()
parser = create_parser(exchange="deribit", is_cpp=len(sys.argv) > 1 and "cpp" in sys.argv)
watcher = BookWatcher(parser)
conn.on_message = parser.parse

logging.basicConfig(format='%(asctime)s %(name)s %(funcName)s %(levelname)s %(message)s', level=logging.INFO)
logger = logging.getLogger("deribit-book")


async def subscribe_book():
    await conn.send_public(request={
        "method": "public/subscribe",
        "params": {
            "channels": [f"book.BTC-PERPETUAL.raw"]
        }
    })


output = open("deribit_dump_btc_perpetual.jsonl", "w")


def dump(msg: str):
    output.write(msg)
    output.write('\n')


conn.on_connect_ws = subscribe_book
conn.on_before_handling = dump
loop = asyncio.get_event_loop()

try:
    loop.run_until_complete(conn.run_receiver())
except KeyboardInterrupt:
    print("Application closed by KeyboardInterrupt.")



#!/usr/bin/env python
import asyncio
import logging

from ssc2ce import Cex

logging.basicConfig(format='%(asctime)s %(name)s %(funcName)s %(levelname)s %(message)s', level=logging.INFO)
logger = logging.getLogger("bitfinex-basic-example")

conn = Cex()


async def subscribe():
    data = await conn.public_get("currency_limits")
    if data.get('ok') == 'ok':
        for pair in data['data']['pairs']:
            if pair['symbol1'] == 'BTC' and pair['symbol2'] == 'EUR':
                print(pair)

    data = await conn.public_get("ticker/BTC/EUR")
    print(data)

    request = {'e': "subscribe", 'rooms': ["ticker"]}

    await conn.ws.send_json(request)


def print_ticker(msg):
    print(msg)
    return True


conn.on_connect_ws = subscribe
conn.on_message = print_ticker

loop = asyncio.get_event_loop()

try:
    loop.run_until_complete(conn.run_receiver())
except KeyboardInterrupt:
    print("Application closed by KeyboardInterrupt.")



#!/usr/bin/env python
import asyncio
import logging

from ssc2ce import Coinbase

logging.basicConfig(format='%(asctime)s %(name)s %(funcName)s %(levelname)s %(message)s', level=logging.INFO)
logger = logging.getLogger("bitfinex-basic-example")


def handle_message(message: str):
    logger.info(message)


conn = Coinbase()
conn.on_message = handle_message


async def subscribe():
    await conn.ws.send_json({
        "type": "subscribe",
        "product_ids": [
            "BTC-USD",
            "ETH-BTC"
        ],
        "channels": [
            "level2",
            "heartbeat"
        ]
    })


def handle_heartbeat(data: dict) -> bool:
    logger.info(f"{repr(data)}")
    return True


conn.on_connect_ws = subscribe


def stop():
    asyncio.ensure_future(conn.stop())


loop = asyncio.get_event_loop()
loop.call_later(3600, stop)

try:
    loop.run_until_complete(conn.run_receiver())
except KeyboardInterrupt:
    logger.info("Application closed by KeyboardInterrupt.")



import json
import logging
from typing import Callable, Dict, Optional

from ssc2ce.common.abstract_parser import AbstractParser
from ssc2ce.deribit.l2_book import DeribitL2Book
from ssc2ce.common.exceptions import BrokenOrderbook


class DeribitParser(AbstractParser):
    """

    """

    def __init__(self):
        AbstractParser.__init__(self)
        self.logger = logging.getLogger(__name__)
        self.deprecated_already_warn = False
        self._handler_broken_orderbook: Optional[Callable[[str, int, int], None]] = None
        self._books: Dict[str, DeribitL2Book] = {}

    async def handle_message(self, message):
        if not self.deprecated_already_warn:
            self.deprecated_already_warn = True
            self.logger.warning("The handle_message method of DeribitParser is deprecated, please use parse instead")
        self.parse(message)

    def parse(self, message) -> bool:
        data = json.loads(message)

        return self.handle_method_message(data)

    def handle_method_message(self, data: dict) -> bool:
        method = data.get("method")
        processed = False
        if method and method == "subscription":
            params = data["params"]
            channel = params["channel"]
            if channel.startswith("book."):
                params_data = params["data"]
                instrument = params_data["instrument_name"]
                book = self.get_book(instrument)

                if "prev_change_id" in params_data:
                    self.handle_update(book, params_data)
                    if self._on_book_update:
                        self._on_book_update(book)
                else:
                    self.handle_snapshot(book, params_data)
                    if self._on_book_setup:
                        self._on_book_setup(book)

                processed = True

        return processed

    def get_book(self, instrument: str) -> DeribitL2Book:
        book = self._books.get(instrument)
        if book is None:
            book = DeribitL2Book(instrument)
            self._books[instrument] = book

        return book

    @staticmethod
    def handle_snapshot(book: DeribitL2Book, message: dict) -> None:
        """

        :param book:
        :param message:
        :return:
        """
        book.clear()

        book.change_id = message["change_id"]
        book.timestamp = message["timestamp"]
        for i in message['bids']:
            book.bids.add(i[1], i[2])

        for i in message['asks']:
            book.asks.add(i[1], i[2])

    def handle_update(self, book: DeribitL2Book, message: dict) -> None:
        """

        :param book:
        :param message:
        :return:
        """
        prev_change_id = message["prev_change_id"]
        if prev_change_id != book.change_id:
            if self._handler_broken_orderbook:
                self._handler_broken_orderbook(book.instrument(), prev_change_id, book.change_id)
            else:
                raise BrokenOrderbook(book.instrument(), prev_change_id, book.change_id)

        book.change_id = message["change_id"]
        book.timestamp = message["timestamp"]
        for change in message['bids']:
            if change[0] == 'new':
                book.bids.add(price=change[1], size=change[2])
            elif change[0] == 'delete':
                book.bids.delete(change[1])
            else:
                book.bids.update(price=change[1], size=change[2])

        for change in message['asks']:
            if change[0] == 'new':
                book.asks.add(price=change[1], size=change[2])
            elif change[0] == 'delete':
                book.asks.delete(change[1])
            else:
                book.asks.update(price=change[1], size=change[2])
